package com.persagy.apm.translate.aop;

import com.google.common.cache.Cache;
import com.google.common.cache.CacheBuilder;
import com.persagy.apm.translate.helper.TranslatorHelper;
import com.persagy.apm.translate.model.ITransable;
import com.persagy.apm.translate.model.Trans;
import com.persagy.apm.translate.model.TransMeta;
import com.persagy.apm.translate.model.Transes;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.core.Ordered;
import org.springframework.stereotype.Component;

import java.lang.reflect.Method;
import java.util.List;

/**
 * 翻译器Aspect
 * @author Charlie Yu
 * @Date 2021-03-03
 */
@Component
@Aspect
@Slf4j
public class TranslateAopAspect implements Ordered {

    /** 缓存池大小 */
    private final int CACHE_COUNT = 100;

    /** 翻译描述信息缓存 */
    private Cache<String,List<TransMeta>> transCach;

    /**
     * 构造方法,初始化缓存
     */
    public TranslateAopAspect(){
        transCach = CacheBuilder.newBuilder().initialCapacity(CACHE_COUNT/2).maximumSize(CACHE_COUNT).build();
    }

    @Pointcut("@annotation(com.persagy.apm.translate.model.Transes)||@annotation(com.persagy.apm.translate.model.Trans)")
    private void pointCutMethod() {
    }

    /**
     * 取得方法缓存的Key
     * @param signature 方法标签
     * @return 缓存Key
     */
    private String getMethodKey(MethodSignature signature){
        return signature.toString();
    }

    /**
     * 获取翻译元数据
     * @param signature 方法标示
     * @throws Exception
     */
    public List<TransMeta> buildMeta(MethodSignature signature) throws Exception {
        String key = getMethodKey(signature);
        List<TransMeta> metas = transCach.getIfPresent(key);
        if(metas!=null) {
            return metas;
        }
        Method method = signature.getMethod();
        Transes transes = method.getAnnotation(Transes.class);
        if(transes != null && transes.value().length > 0) {
            metas = TranslatorHelper.parseMetas(transes.value());
        }else{
            Trans tran = method.getAnnotation(Trans.class);
            if(tran != null) {
                metas = TranslatorHelper.parseMetas(new Trans[]{tran});
            }
        }
        if(metas != null) {
            transCach.put(key,metas);
        }
        return metas;
    }

    @Around("pointCutMethod()")
    public Object process(ProceedingJoinPoint point) throws Throwable {
        try {
            log.info("Around translate---"+point.getSignature().toLongString());
            List<TransMeta> metas = buildMeta((MethodSignature) point.getSignature());
            Object rt = point.proceed();

            if (metas == null || rt == null) {
                return rt;
            }
            if (rt instanceof ITransable) {
                rt = TranslatorHelper.translate((ITransable) rt, metas);
            } else {
                rt = TranslatorHelper.translate(rt,metas);
            }
            log.info("Around translate-complete---"+point.getSignature().toLongString());
            return rt;
        } finally {
            TranslatorHelper.clearMetas();
        }
    }

    @Override
    public int getOrder() {
        return 2000;
    }
}